﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Permissions;
using System.Text;

namespace WinInetDemo3
{
    internal class DownloadManager : IDisposable
    {
        private bool _cancelInstall = false;
        private Stopwatch _downloadStopWatch = null;
        private MyWinInet _winInet = null;
        private int _currentDownloadTotalBytes = 0;
        private int _currentDownloadBytesSinceStopWatch = 0;
        private BackgroundWorker _bgWorker = null;

        internal MyWinInet MyWinInet
        {
            get
            {
                if (this._winInet == null)
                {
                    this._winInet = new MyWinInet();
                    this._winInet.InternetOpen();
                    this._winInet.BytesRead += new EventHandler<MyBytesReadEventArgs>(this.HandleGcWinInet_BytesRead);
                }
                return this._winInet;
            }
        }

        public DownloadManager(BackgroundWorker bgWorker)
        {
            _bgWorker = bgWorker;
            _downloadStopWatch = new Stopwatch();
        }

        public bool DownloadFile(string url, string fileName)
        {
            bool isDownloadSuccessfully = false;
            this._downloadStopWatch.Start();
            try
            {
                Uri installerUrl = new Uri(url);
                string tempFileName = Path.GetTempFileName();

                this._currentDownloadTotalBytes = 0;
                this._currentDownloadBytesSinceStopWatch = 0;
                if (this.MyWinInet.DownloadFile(installerUrl, tempFileName))
                {
                    if (!this.MyWinInet.CancelDownload)
                    {
                        if (File.Exists(fileName))
                        {
                            Util.DeleteFileIfExists(fileName);
                        }
                        DownloadInfo info = new DownloadInfo() { Message = "xxx", Persent = 100, Speed = 0 };
                        DownloadingStatusChanged(info);

                        File.Move(tempFileName, fileName);
                        isDownloadSuccessfully = true;
                    }
                    else
                    {
                        DownloadInfo info = new DownloadInfo() { Message = "已取消下载", Persent = 100, Speed = 0 };
                        DownloadingStatusChanged(info);
                    }
                }   
            }
            catch
            {
                // todo
            }
            finally
            {
                // 计算下载所用时间
                this._downloadStopWatch.Stop();

                if (this._winInet != null)
                {
                    this._winInet.BytesRead -= new EventHandler<MyBytesReadEventArgs>(this.HandleGcWinInet_BytesRead);
                    this._winInet.InternetClose();
                    this._winInet = null;
                }
            }
            return isDownloadSuccessfully;
        }

        public void Cancel()
        {
            this._cancelInstall = true;
            if (this._winInet != null)
            {
                this._winInet.CancelDownload = true;
            }
        }

        private bool InternalDownload(string url, string fileName)
        {
            bool flag = false;
            

            return flag;
        }

        private void HandleGcWinInet_BytesRead(object sender, MyBytesReadEventArgs e)
        {
            this._currentDownloadTotalBytes += e.BytesRead;
            this._currentDownloadBytesSinceStopWatch += e.BytesRead;
            int intDownloadSpeed = 0;
            if (this._downloadStopWatch.ElapsedMilliseconds > 800L)
            {
                double num = (double)this._currentDownloadBytesSinceStopWatch / 1024.0;
                double num2 = (double)this._downloadStopWatch.ElapsedMilliseconds / 1000.0;
                double value = num / num2;
                intDownloadSpeed = (int)Math.Round(value, 0);
                this._downloadStopWatch.Reset();
                this._downloadStopWatch.Start();
                this._currentDownloadBytesSinceStopWatch = 0;
            }
            double num3 = (double)this._currentDownloadTotalBytes / (double)e.TotalBytes;
            if (num3 > 1.0)
            {
                num3 = 1.0;
            }
            int downloadPersent = (int)(num3 * 100);

            if (intDownloadSpeed > 0)
            {
                DownloadInfo info = new DownloadInfo() { Message = "xxx", Persent = downloadPersent, Speed = intDownloadSpeed };
                DownloadingStatusChanged(info);
            }
        }

        [EnvironmentPermission(SecurityAction.Demand, Unrestricted = true)]
        public void Dispose()
        {
            MyWinInet gcWinInet = this._winInet;
            this._winInet = null;
            if (gcWinInet != null)
            {
                gcWinInet.InternetClose();
                gcWinInet.Dispose();
            }
            GC.SuppressFinalize(this);
        }

        private void DownloadingStatusChanged(DownloadInfo info)
        {
            if (_bgWorker != null && info != null)
            {
                _bgWorker.ReportProgress(0, info);
            }
        }
    }

}

